
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace BallDroppingOnXBox
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        private BasicEffect basicEffect;
        private EffectPool effectPool;
        private Matrix worldMatrix, viewMatrix, projectionMatrix;

        // these are the size of the output window, ignored
        // on Xbox 360
        private int preferredWindowWidth = 640;
        private int preferredWindowHeight = 360;

        // these are the size of the offscreen drawing surface
        // in general, no one wants to change these as there
        // are all kinds of UI calculations and positions based
        // on these dimensions.
        const int FixedDrawingWidth = 1280;
        const int FixedDrawingHeight = 720;

        private Circle2D myCircle;
        private float rot;
        private Texture2D myTexture;
        
        private VertexPositionColor[] corners;
        private VertexDeclaration verDec;
        private Stack<Matrix> mStack;


        public Game1()
        {


#if XBOX360
            // we might as well use the xbox in all its glory
            preferredWindowWidth = FixedDrawingWidth;
            preferredWindowHeight = FixedDrawingHeight;
            // enableDrawScaling = false;
#else
            // graphics.enableDrawScaling = true;

            I don't think I support this for now ...
#endif

            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = preferredWindowWidth;
            graphics.PreferredBackBufferHeight = preferredWindowHeight;
            // graphics.PreparingDeviceSettings += new EventHandler<PreparingDeviceSettingsEventArgs>(PreparingDeviceSettings);

            // Game should run as fast as possible.
            IsFixedTimeStep = false;
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            
            //  my object
            mStack = new Stack<Matrix>();
            myCircle = new Circle2D(graphics.GraphicsDevice, new Vector3(5.0f, 3.0f, 0.0f), 1.0f, 100);
            rot = 0.0f;
            verDec = new VertexDeclaration(graphics.GraphicsDevice, VertexPositionColor.VertexElements);

            corners = new VertexPositionColor[4];
            for (int i=0; i<4; i++)
                corners[i].Color = Color.Brown;

            corners[0].Position = new Vector3(0.5f, 0.5f, 0.0f);
            corners[1].Position = new Vector3(0.5f, 7.0f, 0.0f);
            corners[2].Position = new Vector3(12.5f, 7.0f, 0.0f);
            corners[3].Position = new Vector3(12.5f, 0.5f, 0.0f);

            // my device settings
            graphics.GraphicsDevice.RenderState.DepthBufferEnable = false;
            graphics.GraphicsDevice.RenderState.MultiSampleAntiAlias = true;
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.None;
                        
            // render "effect"
            effectPool = new EffectPool();
            basicEffect = new BasicEffect(graphics.GraphicsDevice, effectPool);

            // initialize the transformation pipeline to draw in 12.8x7.2 area
            worldMatrix = new Matrix();
            viewMatrix = new Matrix();
            projectionMatrix = new Matrix();

            Matrix s, t;
            t = Matrix.CreateTranslation(-1.0f, -1.0f, 0.0f);
            s = Matrix.CreateScale(2.0f / 12.8f, 2.0f / 7.2f, 1.0f);
            worldMatrix = Matrix.Identity;
            viewMatrix = s * t;
            projectionMatrix = Matrix.Identity;
    
            // here is the Texture bit ...
            basicEffect.World = worldMatrix;
            basicEffect.View = viewMatrix;
            basicEffect.Projection = projectionMatrix;
            basicEffect.LightingEnabled = false;
            basicEffect.VertexColorEnabled = true;  // Gouraud Shading

            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            base.LoadGraphicsContent(loadAllContent);

            if (loadAllContent)
            {
                content = new ContentManager(Services);

                String name = @"texture\Spacewar";

                // load in the texture
                myTexture = content.Load<Texture2D>(name);
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            base.UnloadGraphicsContent(unloadAllContent);
            if (unloadAllContent == true)
            {
                content.Dispose();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            Vector2 delta = GamePad.GetState(PlayerIndex.One).ThumbSticks.Left;
            float len = delta.LengthSquared();
            if (len > 0.01f)
            {
                delta.Normalize();
                delta *= 0.1f;
            }

            int vib = 0;
            Vector3 c = myCircle.getCenter();
            c.X += delta.X;
            c.Y += delta.Y;

            if (c.X > 12.5)
            {
                c.X = 12.5f;
                vib+=2;
            }
            else
                if (c.X < 0.5f)
                {
                    c.X = 0.5f;
                    vib++;
                }
                    

            if (c.Y < 0.5f)
            {
                c.Y = 0.5f;
                vib++;
            }
            else
                if (c.Y > 7.0)
                {
                    c.Y = 7.0f;
                    vib+=2;
                }
                
            myCircle.setCenter(c);
            if (vib == 0)
                GamePad.SetVibration(PlayerIndex.One, 0.0f, 0.0f);
            else
            {
                if (vib == 1)
                    GamePad.SetVibration(PlayerIndex.One, 1.0f, 0.0f);
                else
                    GamePad.SetVibration(PlayerIndex.One, 1.0f, 1.0f);
            }

            GamePadButtons but = GamePad.GetState(PlayerIndex.One).Buttons;
            if (but.A == ButtonState.Pressed)
            {
                rot += 0.01f;
                if (rot > (2.0 * Math.PI))
                    rot -= (float)(2 * Math.PI);
            }
            else if (but.B == ButtonState.Pressed)
            {
                rot -= 0.01f;
                if (rot < (2.0 * Math.PI))
                    rot += (float)(2 * Math.PI);
            }
            
            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            
            base.Draw(gameTime);

            basicEffect.Texture = myTexture;
            basicEffect.TextureEnabled = true;


            // for now, I think the basicEffect is meant to be associated with each Primitive object
            basicEffect.Begin();
            basicEffect.Techniques[0].Passes[0].Begin();
            basicEffect.CommitChanges();

                    // Draw the top-left large viewport ...
                    Viewport v = new Viewport();
                    v.X = 20;  // Viewport x/y identifies top-left corner
                    v.Y = 20;
                    v.Width = (int)(FixedDrawingWidth * 0.75f);
                    v.Height = (int)(FixedDrawingHeight * 0.75f);
                    v.MaxDepth = 1.0f;
                    v.MinDepth = -1.0f;
                    graphics.GraphicsDevice.Viewport = v;


                    graphics.GraphicsDevice.Clear(Color.CornflowerBlue);


                        // corner for the circle
                        graphics.GraphicsDevice.VertexDeclaration = verDec; 
                        graphics.GraphicsDevice.RenderState.PointSize = 10.0f;
                        graphics.GraphicsDevice.DrawUserPrimitives(PrimitiveType.PointList, corners, 0, 4);

                        mStack.Push(basicEffect.World);
                        // do the drawing of each primitives ...
                        {
                            Vector3 c = myCircle.getCenter();
                            Matrix r, t, tb;
                            t = Matrix.CreateTranslation(-c.X, -c.Y, 0.0f);
                            tb = Matrix.CreateTranslation(c.X, c.Y, 0.0f);
                            r = Matrix.CreateRotationZ(rot);
                            basicEffect.World = t * r * tb;
                            basicEffect.CommitChanges();
                        }
                        myCircle.Draw(graphics.GraphicsDevice);

                        basicEffect.World = mStack.Pop();
                        basicEffect.CommitChanges();

                 // ============= SECOND VIEWPORT DRAWING ========================

                        //  THis is cut and paste from above (think abstraction!!)
                        // Draw the top-left large viewport ...
                        Viewport v2 = new Viewport();
                        v2.X = (int)(FixedDrawingWidth * 0.65f);  // Viewport x/y identifies top-left corner
                        v2.Y = (int)(FixedDrawingHeight * 0.65f); ;
                        v2.Width = (int)(FixedDrawingWidth * 0.25f);
                        v2.Height = (int)(FixedDrawingHeight * 0.25f);
                        v2.MaxDepth = 1.0f;
                        v2.MinDepth = -1.0f;
                        graphics.GraphicsDevice.Viewport = v2;


                        graphics.GraphicsDevice.Clear(Color.Cyan);

                         // Since the circle is defined in the WC, notice the following is IDENTICAL to above
                        // corner for the circle
                        graphics.GraphicsDevice.VertexDeclaration = verDec;
                        graphics.GraphicsDevice.RenderState.PointSize = 10.0f;
                        graphics.GraphicsDevice.DrawUserPrimitives(PrimitiveType.PointList, corners, 0, 4);

                        mStack.Push(basicEffect.World);
                        // do the drawing of each primitives ...
                        {
                            Vector3 c = myCircle.getCenter();
                            Matrix r, t, tb;
                            t = Matrix.CreateTranslation(-c.X, -c.Y, 0.0f);
                            tb = Matrix.CreateTranslation(c.X, c.Y, 0.0f);
                            r = Matrix.CreateRotationZ(rot);
                            basicEffect.World = t * r * tb;
                            basicEffect.CommitChanges();
                        }
                        myCircle.Draw(graphics.GraphicsDevice);

                        basicEffect.World = mStack.Pop();


            basicEffect.Techniques[0].Passes[0].End();
            basicEffect.End();

        }
    }
}